# -*- coding: utf-8 -*-
"""
@author: João Ferreira
"""
import nltk.data
import os
from nltk.corpus import floresta
from nltk.tokenize import sent_tokenize,word_tokenize
from nltk.corpus.reader import TaggedCorpusReader
from nltk.tokenize import LineTokenizer
from nltk.corpus import treebank
from nltk.metrics import accuracy
from nltk.corpus import machado
import pickle
import nltk
import time
import xmltodict
from pathlib import Path
start_path = Path(__file__).parent.absolute()

def load_tag_configurations(config_file):
	model_path = ""
	with open(config_file) as g:
		for line in g:
			if(line[0]!="#"):
				if(line.split("=")[0]=="model"):
					model_path = start_path.joinpath(line.split("=")[1].strip('\n'))
	return model_path

def load_tagger(model_path):
	f = open(model_path, 'rb')
	return (pickle.load(f))


def nlpyport_pos(token,config_file):
	model_path = ""
	model_path = load_tag_configurations(config_file)
	tagger = load_tagger(model_path)
	tags = [tagger.tag(token)]
	i = 0
	result_tags = [None] * len(token)
	result_toks = [None] * len(token)
	for tag in tags:
		for elem in tag:
			if(len(elem[1].split(":"))>1):
				result_tags[i] = elem[1].split(":")[1]
			else:
				result_tags[i] = elem[1]
			i += 1
	return result_tags, tags


if __name__ == "__main__":
	start_time = time.time()



from NLPyPort.LemPyPort.normalization.verb_normalizer import *
from NLPyPort.LemPyPort.normalization.superlative_normalizer import *
from NLPyPort.LemPyPort.normalization.number_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_name_normalizer import *
from NLPyPort.LemPyPort.normalization.diminutive_normalizer import *
from NLPyPort.LemPyPort.normalization.augmentative_normalizer import *
from NLPyPort.LemPyPort.normalization.adverb_normalizer import *



import xmltodict
import re

class adverb_normalizer:
	def __init__(self):

		self.declesion_exceptions=[]
		self.declesion_targets = []
		self.declesion_tags = []
		self.declesion_replacement = []
		self.declesion_prefix = []
		self.declesion_re = []

	def compile_rules(self):
		lista = [None] * len(self.declesion_targets)
		for index,elem in enumerate(self.declesion_targets):
			li = re.compile(self.declesion_prefix+elem)
			lista[index] = li
		self.declesion_re=lista


	def adverb_normalizer_load(self, file_name):
		with open(file_name) as file:
			doc = xmltodict.parse( file.read())
			result = (doc["adverbdeclensions"]["replacement"])

			genderdeclensions_number_of_rules = len(result)
			self.declesion_exceptions = [None]*genderdeclensions_number_of_rules
			self.declesion_targets = [None]*genderdeclensions_number_of_rules
			self.declesion_tags = [None]*genderdeclensions_number_of_rules
			self.declesion_replacement = [None]*genderdeclensions_number_of_rules

			for index, elem in enumerate(result):
				if '@target' in elem.keys():
					self.declesion_targets[index] = elem["@target"]
				else:
					self.declesion_targets[index] = ""
				if '@exceptions' in elem.keys():
					self.declesion_exceptions[index] = elem["@exceptions"]
				else:
					self.declesion_exceptions[index] = ""
				if '@tag' in elem.keys():
					self.declesion_tags[index] = elem["@tag"]
				else:
					self.declesion_tags[index] = ""
				if '#text' in elem.keys():
					self.declesion_replacement[index] = elem["#text"]
				else:
					self.declesion_replacement[index] = ""
			self.declesion_prefix = (doc["adverbdeclensions"]["prefix"])					

	def print_adverb_normalizer(self):
		for i in range(len(self.declesion_targets)):
			print(i)
			print("---------\n")
			print("Target: " + self.declesion_targets[i])
			print("Tags: " + self.declesion_tags[i])
			print("Exceptions: " + self.declesion_exceptions[i])
			print("Replacement: " + self.declesion_replacement[i])

	def normalize_adverb(self,token, tag):
		bigget_rule_size = 0
		lemmatized_word = ""
		normalization = token.lower()
		for index,elem in enumerate(self.declesion_targets):
			if(self.declesion_re[index].match(normalization)
					and tag.lower() in self.declesion_tags[index].split("|")
						and not (normalization in self.declesion_exceptions[index].split("|"))):
					tamanho = len(self.declesion_targets[index])
					if(tamanho > bigget_rule_size):
						lemmatized_word = normalization[:(len(normalization) - len(self.declesion_targets[index]))] + self.declesion_replacement[index]
						bigget_rule_size = tamanho
						return(lemmatized_word)
		return(lemmatized_word)




# -*- coding: utf-8 -*-
"""
@author: João Ferreira
"""
import nltk.data
import os
from nltk.corpus import floresta
import nltk
import xmltodict
from pathlib import Path

start_path = Path(__file__).parent.parent.absolute()

def load_token_configurations(config_file):
	contractions_path = ""
	clitics_path = ""
	with open(config_file) as g:
		for line in g:
			if(line[0]!="#"):
				if(line.split("=")[0]=="contractions"):
					contractions_path = start_path.joinpath(line.split("=")[1].strip('\n'))
				elif(line.split("=")[0]=="clitics"):
					clitics_path = start_path.joinpath(line.split("=")[1].strip('\n'))
	return contractions_path,clitics_path

def get_input_from_file(fileinput):
	text = []
	with open(fileinput,'r') as f:
		for line in f:
			text .append(line)
	return text

def replace_contrations(contractions_path,tokens):
	tokens_after_contractions = []
	encontrou = 0

	#Check if tokens contain contractions
	#If so, change them to the most extended form
	with open(contractions_path) as fd:
		doc = xmltodict.parse(fd.read())
		result = (doc["contractions"]["replacement"])
		for tok in tokens:
			encontrou = 0
			token2 = tok
			for elem in result:
				if(tok==elem['@target']):
					encontrou=1
					subs = elem['#text'].split(" ")
					for part in subs:
						tokens_after_contractions.append(part)
			#if word in not contration add it as it was
			if(encontrou==0):
				tokens_after_contractions.append(token2)
	return tokens_after_contractions

def replace_clitics(clitics_path,tokens):
	tokens_after_clitics =[]
	with open(clitics_path) as fd:
		doc2 = xmltodict.parse(fd.read())
		result2 = (doc2["clitics"]["replacement"])
		for tok2 in tokens:
			if(len(tokens)>0):
				encontrou = 0
				token2 = tok2
				for elem2 in result2:
					if(tok2==elem2['@target']):
						encontrou=1
						subs = elem2['#text'].split(" ")
						for part in subs:
							tokens_after_clitics.append(part)
				if(encontrou==0):
					withslash = tok2.split("-")
					if(len(withslash)>1):
						nova_palavra = ""
						for parte in withslash:
							if(parte!=withslash[0]):
								nova_palavra +="-" + parte
						encontrou = 0
						token2 = tok2
						for elem2 in result2:
							if(nova_palavra==elem2['@target']):
								encontrou=1
								subs = elem2['#text'].split(" ")
								for part in subs:
									tokens_after_clitics.append(part)
						#if word in not contration add it as it was
						if(encontrou==1):
							tokens_after_clitics.append(withslash[0]+"-")
				if(encontrou==0):
					tokens_after_clitics.append(token2)
			else:
				tokens_after_clitics.append(tokens)
	return tokens_after_clitics	

def nltk_tokenize(text):
	result = []
	for line in text:
		tok=(nltk.word_tokenize(line))
		for elem in tok:
			result.append(elem)
		result.append("EOS")
	return result

def nlpyport_tokenize_from_string(text,TokPort_config_file):
	#define the tagset being used
	#print(text)
	floresta.tagged_words(tagset = "pt-bosque")
	contractions_path = ""
	clitics_path = ""
	tokens=[]
	tokens_after_contractions = []
	tokens_after_clitics = []
	#get the directory of the resources
	contractions_path,clitics_path = load_token_configurations(TokPort_config_file)
	text_list = []
	if(isinstance(text, str)):
		text = text.replace("\n"," EOS")
		if("EOS" not in text):
			text+=" EOS"
		text_list = [text]

		#print(text)
	else:
		current_text = ""
		for elem in text:
			elem = elem.replace("\n"," EOS\n")
			has_eof = 0
			if("EOS" not in elem):
				elem+=" EOS\n"
			if elem != '\n':
				current_text += elem
		if(current_text!=""):
			text_list.append(current_text)
	
	text = text_list
	#Do the actual tokenization
	#print(str(text_list))
	tokens = nltk_tokenize(text)

	#print(tokens)
	novos_tokens = []
	for i in range(len(tokens)):
		if(tokens[i]!="\n"):
			novos_tokens.append(tokens[i])
	tokens = novos_tokens
	tokens_after_contractions = replace_contrations(contractions_path,tokens)

	#Check if tokens contain clitics
	#If so, change them to the most extended form
	tokens_after_clitics =replace_clitics(clitics_path,tokens_after_contractions)
	final_tokens =[]
	if not(isinstance(text, str)):
		for tok in tokens_after_clitics[:-1]:
			final_tokens.append(tok)
	else:
		for tok in tokens_after_clitics:
			final_tokens.append(tok)
	return final_tokens

def nlpyport_tokenizer(fileinput,TokPort_config_file):
	#define the tagset being used
	floresta.tagged_words(tagset = "pt-bosque")
	contractions_path = ""
	clitics_path = ""
	text = " "
	tokens=[]
	tokens_after_contractions = []
	tokens_after_clitics = []
	#get the directory of the resources
	contractions_path,clitics_path = load_token_configurations(TokPort_config_file)

	#Get tokens from input file, one by line
	text = get_input_from_file(fileinput)

	#Do thea actual tokenization
	tokens = nltk_tokenize(text)
	#for i in range(len(tokens)):
	#	if(tokens[i])=="#":
	#		tokens[i] = "\n"
	tokens_after_contractions = replace_contrations(contractions_path,tokens)

	#Check if tokens contain clitics
	#If so, change them to the most extended form
	tokens_after_clitics =replace_clitics(clitics_path,tokens_after_contractions)
	final_tokens =[]
	for tok in tokens_after_clitics:
		final_tokens.append(tok)
	return final_tokens

'''
if __name__ == '__main__':
	print(nlpyport_tokenizer("EntradaCadeiaTotal.txt"))
'''



from NLPyPort.LemPyPort.dictionary.dictionary_entry import *
class dictionary:
    def __init__(self):
        self.dictionary_list = {}

    def load(self,input_file):
        inflected_form = ""
        lemma = ""
        part_of_speech = ""
        subcategory = ""
        morph_attributes = ""
        with open(input_file,'r') as f:
            for line in f:
                if(len(line)>0 and not (line[0]=="#")):
                    if(("." in line and "," in line) \
                    and (line.find(".")>(line.find(",")+1)) \
                    and (line.find(".") +1<len(line)) \
                    and (not (line[line.find(".")+1] == ":") \
                        or (not line[line.find(".")+1] == "+"))):
                        #print(line)
                        inflected_form = line[0:line.find(",")]
                        #print("inflected:" + inflected_form)
                        lemma = line[line.find(",")+1:line.find(".")]
                        #print("lemma:" + lemma)

                        if("+" in line):
                            part_of_speech = line[line.find(".")+1:line.find("+")]
                            #print("Pos1: " + part_of_speech)
                        elif (":" in line):
                            part_of_speech = line[line.find(".")+1:line.find(":")]
                            #print("Pos2: " + part_of_speech)
                        else:
                            part_of_speech =  line[line.find(".")+1:]
                            #print("Pos3: " + part_of_speech)
                        if("+" in line):
                            if(":" in line):
                                subcategory = line[line.find("+")+1:line.find(":")]
                                #print("Subcat1: " + subcategory)
                            else :
                                subcategory = line[line.find("+"):]
                                #print("Subcat2: " + subcategory)
                        entry = dictionary_entry(inflected_form,lemma,part_of_speech,subcategory,morph_attributes)
                        #if(entry.inflected_form=="abelha" and entry.part_of_speech=="V"):
                        #    print("---Ok---")
                        self.add(entry)
                        #print(new_entry)
                        #entry.print_entry()
                        #self.print_dictionary_list()
                        #print(self.contains("murchos"))
                        #print(self.contains("murchos","V"))
                        #print(self.retrive_lemas("murchos","V"))
                        #print(self.retrive_entries("murchos"))
        #print(self.remove("murchos","V"))
        #print("------")
        #print(self.remove("murchos","p"))
        #print(self.remove("murchos"))

    def add(self,entry):
        entry_set = self.dictionary_list.get(entry.inflected_form)
        if (entry_set == None ):
            entry_set = []
        entry_set.append(entry)
        self.dictionary_list[entry.inflected_form] = entry_set

    def remove(self,inflected_form):
        removed_entries = self.dictionary_list.pop(inflected_form,None)
        if(removed_entries is not None):
            return removed_entries
        return None

    def remove(self,inflected_form,part_of_speech):
        entry_set = []
        entry_set.append(self.dictionary_list.pop(inflected_form,None))
        #print(entry_set[0].inflected_form)
        remaining_entries = []
        removed_entries = []
        if(entry_set[0] is not None):
            for elem in entry_set:
                if((elem.inflected_form == inflected_form)\
                and elem.part_of_speech == part_of_speech):
                    removed_entries.append(elem)
                else:
                    remaining_entries.append(elem)
            if(len(remaining_entries)>0):
                self.dictionary_list[inflected_form]=remaining_entries[0]
            if(len(removed_entries)>0):
                return removed_entries[0]
        else:
            return None



    def print_dictionary_list(self):
        for elem in self.dictionary_list:
            print(elem+ " " +str(self.dictionary_list[elem].print_entry()))
            

    def contains(self,inflected_form):
        return inflected_form in self.dictionary_list

    def contains(self,inflected_form,part_of_speech):
        if(inflected_form in self.dictionary_list):
            entry = self.dictionary_list[inflected_form]
            for elem in entry:
                if(part_of_speech in elem.part_of_speech):
                    return True
        return False
    
    def retrive_lemas(self,inflected_form,part_of_speech):
        lemas=[]
        if(inflected_form in self.dictionary_list):
            entry = self.dictionary_list[inflected_form]
            for elem in entry:
                if(part_of_speech in elem.part_of_speech):
                    lemas.append(elem.lemma)
        return lemas

    def retrive_entries(self,inflected_form):
        entries=[]
        if(inflected_form in self.dictionary_list):
            entry = self.dictionary_list[inflected_form]
            for elem in entry:
                if(not(elem == [])):
                    entries.append(elem.inflected_form)
        return entries

    '''
      public DictionaryEntry[] retrieveEntries(String inflectedForm) {
        HashSet<DictionaryEntry> entries = new HashSet<DictionaryEntry>();
        if (dictionary.containsKey(inflectedForm)) {
        entries = dictionary.get(inflectedForm);
        }
        return entries.toArray(new DictionaryEntry[entries.size()]);
    }
    '''
    #retrieve_all_entries

    # retrive_lexicon

if __name__ == "__main__":
    novo_dict = dictionary()
    novo_dict.load("Label-Delaf_pt_v4_1.dic")
    print(novo_dict.retrive_lemas("abelha","N"))
    print(novo_dict.contains("abelha","N"))
    print(novo_dict.retrive_lemas("abelha","V"))
    print(novo_dict.retrive_entries("abelha"))
    print(novo_dict.contains("abelha","V"))


from NLPyPort.FullPipeline import *
import sys

#########################
#Example for a file input
#########################

options = {
			"tokenizer" : True,
			"pos_tagger" : True,
			"lemmatizer" : True,
			"entity_recognition" : True,
			"np_chunking" : True,
			"pre_load" : False,
			"string_or_array" : False
}

input_file="input_sample.txt"
text=new_full_pipe(input_file, options=options)
if(text!=0):
		text.print_conll()

'''
#########################
#Example for an array input
#########################

options = {
			"tokenizer" : True,
			"pos_tagger" : True,
			"lemmatizer" : True,
			"entity_recognition" : True,
			"np_chunking" : True,
			"pre_load" : False,
			"string_or_array" : True
}

input_array = ['Primeira frase', 'Segunda frase', 'Terceira frase']
text=new_full_pipe(input_array,options=options)
if(text!=0):
		text.print_conll()

#########################
#Example for an string input
#########################
options = {
			"tokenizer" : True,
			"pos_tagger" : True,
			"lemmatizer" : True,
			"entity_recognition" : True,
			"np_chunking" : True,
			"pre_load" : False,
			"string_or_array" : True
}

input_string = "String para teste"
text=new_full_pipe(input_string,options=options)

if(text!=0):
		text.print_conll()


#########################
#Pipeline with pre-load example 
#########################
options = {
			"tokenizer" : True,
			"pos_tagger" : True,
			"lemmatizer" : True,
			"entity_recognition" : True,
			"np_chunking" : True,
			"pre_load" : False,
			"string_or_array" : True
}
config_list = load_congif_to_list() #This pre-loads the program, makes the start slower but it's faster for larger text
input_string = "String para teste"
text=new_full_pipe(input_string,options=options,config_list=config_list)

if(text!=0):
		text.print_conll()
'''





# -*- coding: utf-8 -*-
"""
@author: João Ferreira
"""
import nltk.data
import os
from nltk.corpus import floresta
from nltk.tokenize import sent_tokenize,word_tokenize
from nltk.corpus.reader import TaggedCorpusReader
from nltk.tokenize import LineTokenizer
from nltk.corpus import treebank
from nltk.metrics import accuracy
from nltk.corpus import machado
from operator import itemgetter
import pickle
import nltk
import time
import xmltodict
import re
from NLPyPort.LemPyPort.normalization.adverb_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_normalizer import *					
from NLPyPort.LemPyPort.normalization.number_normalizer import *
from NLPyPort.LemPyPort.normalization.gender_name_normalizer	import *
from NLPyPort.LemPyPort.normalization.augmentative_normalizer import *
from NLPyPort.LemPyPort.normalization.diminutive_normalizer import *
from NLPyPort.LemPyPort.normalization.superlative_normalizer import *
from NLPyPort.LemPyPort.normalization.verb_normalizer import *
from NLPyPort.LemPyPort.dictionary.dictionary_entry import *
from NLPyPort.LemPyPort.dictionary.dictionary import *
from NLPyPort.LemPyPort.rank.word_ranking import *
from pathlib import Path

start_path = Path(__file__).parent.absolute()
adverb_path = ""
number_path = ""
superlative_path = ""
augmentative_path = ""
diminutive_path = ""
gender_path = ""
gender_name_path = ""
irregular_verb_path = ""
lexeme_verb_path = ""
regular_verb_path  = ""
ranking_path = ""
dict_path = ""
lexical_conversions = ""
dict_exclusions = ""
run_modules = ""
break_on_hyphen = ""
break_on_underscore = ""
dictionary_main_path = ""
custom_dictionary_path = ""


def all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,token,tag,ranking,novo_dict):
	adverb_tags,augmentative_tags,diminutive_tags,gender_name_tags,gender_tags,number_tags,superlative_tags,verb_tags = tags_classes()
	global run_modules
	global break_on_hyphen
	global break_on_underscore
	flags = run_modules
	if(len(token)==0 or len(tag)==0):
		return token
	lemma = token.lower()
	#simplify pos
	result = token
	lex_tag = tag.upper()
	if("-" in lex_tag):
		lex_tag = lex_tag[:lex_tag.find("-")]

	#make translation
	conversion_key = lexical_conversions.split(";")
	for converted_tag in conversion_key:
		if(lex_tag == converted_tag.split(":")[0]):
			lex_tag = converted_tag.split(":")[1]


	if(break_on_hyphen=="True" and "-" in lemma):
		lema1 = lemma[0:lemma.find("-")]
		lema2= lemma[lemma.find("-")+1:]
		res_lema_one = ""
		res_lema_two = ""
		if(lema1!="-"):
			res_lema_one = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,lema1,tag,ranking,novo_dict)
		if(lema2!="-"):
			res_lame_two = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,lema2,tag,ranking,novo_dict)
		return (res_lema_one+"-" + res_lema_two)
			
	if(break_on_underscore=="True" and "_" in lemma):
		lema1 = lemma[0:lemma.find("_")]
		lema2= lemma[lemma.find("_")+1:]
		res_lema_one = ""
		res_lema_two = ""
		if(lema1!="-"):
			res_lema_one = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,lema1,tag,ranking,novo_dict)
		if(lema2!="-"):
			res_lame_two = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,lema2,tag,ranking,novo_dict)
		return (res_lema_one+"_" + res_lema_two)

	if(flags[0]=="1" and (tag in adverb_tags.split("|"))):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_adverb = adverb_norm.normalize_adverb(result,tag)
		if(result_adverb!=""):
			result=result_adverb
	if(flags[1]=="1" and (tag in number_tags.split("|")) ):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)

		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_number = number_norm.normalize_number(result,tag)
		if(result_number!=""):
			result=result_number

	if(flags[2]=="1" and (tag in superlative_tags.split("|")) ):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_superlative = superlative_norm.normalize_superlative(result,tag)
		if(result_superlative!=""):
			result=result_superlative
	if(flags[3]=="1" and (tag in augmentative_tags.split("|")) ):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_augmentative = augmentative_norm.normalize_augmentative(result,tag)
		if(result_augmentative!=""):
			result=result_augmentative
	if(flags[4]=="1" and (tag in diminutive_tags.split("|")) ):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_diminutive = diminutive_norm.normalize_diminutive(result,tag)
		if(result_diminutive!=""):
			result=result_diminutive
	if(flags[5]=="1" and (tag in gender_tags.split("|"))):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		#check dictionary
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_gender = gender_norm.normalize_gender(result,tag)
		if(result_gender!=""):
			result=result_gender
	if(flags[6]=="1" and (tag in gender_name_tags.split("|")) ):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_gender_name = gender_name_norm.normalize_gender_name(result,tag)
		if(result_gender_name!=""):
			result=result_gender_name
	if(flags[7]=="1" and (tag in verb_tags.split("|"))):
		#check dictionary
		if(len(result)>0):
			if(result[-1]=="-"):
				result=result[:-1]
		lema_dict = novo_dict.retrive_lemas(result,lex_tag)
		if(lema_dict!=[] and (lex_tag not in dict_exclusions.split("|"))):
			res = ranking.retrieve_top_word(lema_dict)
			if(not(res==None)):
				return ranking.retrieve_top_word(lema_dict)

		result_verb = verb_norm.normalize_verb(result,tag)
		if(result_verb!=""):
			result=result_verb
	return result


def target_sorter(valores,lista): 
	valores_dic = {}
	desconta_valores = [0] * len(valores)
	for i in range(len(valores)):
		valores_dic[i] = valores[i]
		res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",valores[i])
		if(len(res)>0):
			for elem in res:
				desconta_valores[i] += len(elem)-2
	indices =[]
	new_list = [0] * len(valores)
	place = 0
	for k in sorted(valores_dic, key=lambda k: (len(valores_dic[k])-desconta_valores[k]),reverse = True):
		new_list[place] = lista[k]
		place +=1 
	return(new_list)

def get_paths_lematizador(config_file ):
	global adverb_path
	global number_path
	global superlative_path
	global augmentative_path
	global diminutive_path
	global gender_path
	global gender_name_path
	global irregular_verb_path
	global lexeme_verb_path
	global regular_verb_path 
	global ranking_path 
	global dict_path 
	global lexical_conversions 
	global dict_exclusions 
	global run_modules 
	global break_on_hyphen 
	global break_on_underscore 
	global dictionary_main_path 
	global custom_dictionary_path 

	with open( config_file ,'r') as f:
		for line in f:
			if(line[0] != "#"):
				divided_line = line.split("=")
				if(divided_line[0]=="adverb_path"):
					adverb_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="number_path"):
					number_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="superlative_path"):
					superlative_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="augmentative_path"):
					augmentative_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="diminutive_path"):
					diminutive_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="gender_path"):
					gender_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="gender_name_path"):
					gender_name_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="irregular_verb_path"):
					irregular_verb_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="lexeme_verb_path"):
					lexeme_verb_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="regular_verb_path"):
					regular_verb_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="ranking_path"):
					ranking_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="dict_path"):
					dict_path=start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="lexical_conversions"):
					lexical_conversions=divided_line[1].strip('\n')

				elif(divided_line[0]=="dict_exclusions"):
					dict_exclusions=divided_line[1].strip('\n')

				elif(divided_line[0]=="run_modules"):
					run_modules=divided_line[1].strip('\n')

				elif(divided_line[0]=="break_on_hyphen"):
					break_on_hyphen=divided_line[1].strip('\n')

				elif(divided_line[0]=="break_on_underscore"):
					break_on_underscore=divided_line[1].strip('\n')

				elif(divided_line[0]=="dictionary_main_path"):
					dictionary_main_path = start_path.joinpath(divided_line[1].strip('\n'))

				elif(divided_line[0]=="custom_dictionary_path"):
					custom_dictionary_path = start_path.joinpath(divided_line[1].strip('\n'))

def load_lematizador():

	adverb_norm = adverb_normalizer()
	adverb_norm.adverb_normalizer_load(adverb_path)
	adverb_norm.declesion_exceptions = target_sorter(adverb_norm.declesion_targets,adverb_norm.declesion_exceptions)
	adverb_norm.declesion_tags = target_sorter(adverb_norm.declesion_targets,adverb_norm.declesion_tags)
	adverb_norm.declesion_replacement = target_sorter(adverb_norm.declesion_targets,adverb_norm.declesion_replacement)
	adverb_norm.declesion_targets = target_sorter(adverb_norm.declesion_targets,adverb_norm.declesion_targets)
	adverb_norm.compile_rules()

	number_norm = number_normalizer()
	number_norm.number_normalizer_load(number_path)
	number_norm.declesion_exceptions = target_sorter(number_norm.declesion_targets,number_norm.declesion_exceptions)
	number_norm.declesion_tags = target_sorter(number_norm.declesion_targets,number_norm.declesion_tags)
	number_norm.declesion_replacement = target_sorter(number_norm.declesion_targets,number_norm.declesion_replacement)
	number_norm.declesion_targets = target_sorter(number_norm.declesion_targets,number_norm.declesion_targets)
	number_norm.compile_rules()

	superlative_norm = superlative_normalizer()
	superlative_norm.superlative_normalizer_load(superlative_path)
	superlative_norm.declesion_exceptions = target_sorter(superlative_norm.declesion_targets,superlative_norm.declesion_exceptions)
	superlative_norm.declesion_tags = target_sorter(superlative_norm.declesion_targets,superlative_norm.declesion_tags)
	superlative_norm.declesion_replacement = target_sorter(superlative_norm.declesion_targets,superlative_norm.declesion_replacement)
	superlative_norm.declesion_targets = target_sorter(superlative_norm.declesion_targets,superlative_norm.declesion_targets)
	superlative_norm.compile_rules()
	
	augmentative_norm = augmentative_normalizer()
	augmentative_norm.augmentative_normalizer_load(augmentative_path)
	augmentative_norm.declesion_exceptions = target_sorter(augmentative_norm.declesion_targets,augmentative_norm.declesion_exceptions)
	augmentative_norm.declesion_tags = target_sorter(augmentative_norm.declesion_targets,augmentative_norm.declesion_tags)
	augmentative_norm.declesion_replacement = target_sorter(augmentative_norm.declesion_targets,augmentative_norm.declesion_replacement)
	augmentative_norm.declesion_targets = target_sorter(augmentative_norm.declesion_targets,augmentative_norm.declesion_targets)
	augmentative_norm.compile_rules()

	diminutive_norm = diminutive_normalizer()
	diminutive_norm.diminutive_normalizer_load(diminutive_path)
	diminutive_norm.declesion_exceptions = target_sorter(diminutive_norm.declesion_targets,diminutive_norm.declesion_exceptions)
	diminutive_norm.declesion_tags = target_sorter(diminutive_norm.declesion_targets,diminutive_norm.declesion_tags)
	diminutive_norm.declesion_replacement = target_sorter(diminutive_norm.declesion_targets,diminutive_norm.declesion_replacement)
	diminutive_norm.declesion_targets = target_sorter(diminutive_norm.declesion_targets,diminutive_norm.declesion_targets)
	diminutive_norm.compile_rules()

	gender_norm = gender_normalizer()
	gender_norm.gender_normalizer_load(gender_path)
	gender_norm.declesion_exceptions = target_sorter(gender_norm.declesion_targets,gender_norm.declesion_exceptions)
	gender_norm.declesion_tags = target_sorter(gender_norm.declesion_targets,gender_norm.declesion_tags)
	gender_norm.declesion_replacement = target_sorter(gender_norm.declesion_targets,gender_norm.declesion_replacement)
	gender_norm.declesion_targets = target_sorter(gender_norm.declesion_targets,gender_norm.declesion_targets)
	gender_norm.compile_rules()

	gender_name_norm = gender_name_normalizer()
	gender_name_norm.gender_name_normalizer_load(gender_name_path)
	gender_name_norm.declesion_exceptions = target_sorter(gender_name_norm.declesion_targets,gender_name_norm.declesion_exceptions)
	gender_name_norm.declesion_tags = target_sorter(gender_name_norm.declesion_targets,gender_name_norm.declesion_tags)
	gender_name_norm.declesion_replacement = target_sorter(gender_name_norm.declesion_targets,gender_name_norm.declesion_replacement)
	gender_name_norm.declesion_targets = target_sorter(gender_name_norm.declesion_targets,gender_name_norm.declesion_targets)


	verb_norm = verb_normalizer()
	verb_norm.verb_normalizer_load(irregular_verb_path,lexeme_verb_path,regular_verb_path)
	
	verb_norm.declesion_exceptions = target_sorter(verb_norm.declesion_targets,verb_norm.declesion_exceptions)
	verb_norm.declesion_tags = target_sorter(verb_norm.declesion_targets,verb_norm.declesion_tags)
	verb_norm.declesion_replacement = target_sorter(verb_norm.declesion_targets,verb_norm.declesion_replacement)
	verb_norm.declesion_targets = target_sorter(verb_norm.declesion_targets,verb_norm.declesion_targets)
	
	verb_norm.lexeme_exceptions = target_sorter(verb_norm.lexeme_targets,verb_norm.lexeme_exceptions)
	verb_norm.lexeme_tags = target_sorter(verb_norm.lexeme_targets,verb_norm.lexeme_tags)
	verb_norm.lexeme_replacement = target_sorter(verb_norm.lexeme_targets,verb_norm.lexeme_replacement)
	verb_norm.lexeme_targets = target_sorter(verb_norm.lexeme_targets,verb_norm.lexeme_targets)

	verb_norm.conjugation_exceptions = target_sorter(verb_norm.conjugation_targets,verb_norm.conjugation_exceptions)
	verb_norm.conjugation_tags = target_sorter(verb_norm.conjugation_targets,verb_norm.conjugation_tags)
	verb_norm.conjugation_replacement = target_sorter(verb_norm.conjugation_targets,verb_norm.conjugation_replacement)
	verb_norm.conjugation_targets = target_sorter(verb_norm.conjugation_targets,verb_norm.conjugation_targets)
	
	verb_norm.compile_rules()

	#print("Lemmatizer load completed")
	

	ranking = word_ranking()
	ranking.load(ranking_path)

	
	novo_dict = dictionary()
	novo_dict.load(dictionary_main_path)
	novo_dict.load(custom_dictionary_path)
	#ranking.load("Resources/acdc/formas.total.txt")

	#print("Dictionary load completed")

	return adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict

def tags_classes():
	adverb_tags = "adv"
	augmentative_tags = "n|n-adj|adj"
	diminutive_tags = "n|n-adj|adj"
	gender_name_tags = "n|n-adj|adj"
	gender_tags = "art|pron|pron-pers|pron-det|pron-indp|n|n-adj|adj|num"
	number_tags = "n|n-adj|adj|art|pron|pron-pers|pron-det|pron-indp"
	superlative_tags = "n|n-adj|adj"
	verb_tags = "v|v-fin|v-ger|v-pcp|v-inf"

	return adverb_tags,augmentative_tags,diminutive_tags,gender_name_tags,gender_tags,number_tags,superlative_tags,verb_tags

def nlpyport_lematizer_loader(LemPort_config_file):
	get_paths_lematizador(LemPort_config_file)
	adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict  = load_lematizador()
	return adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict



def nlpyport_lematizer_preload(token,tag,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict):
	valor = []
	for i in range(len(token)):
		res = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,token[i].lower(),tag[i].lower(),ranking,novo_dict)
		valor.append(res)
	return valor


def nlpyport_lematizer(token,tag,LemPort_config_file):
	get_paths_lematizador(LemPort_config_file)
	adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict  = load_lematizador()
	valor = []
	for i in range(len(token)):
		res = all_normalizations(adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,token[i].lower(),tag[i].lower(),ranking,novo_dict)
		valor.append(res)
	return valor



class Text:
	def __init__(self):
		self.tokens = []
		self.pos_tags = []
		self.lemas = []
		self.entities = []
		self.np_tags = []

	def set_tokens(self,tokens):
		self.tokens = tokens


	def set_pos_tags(self,pos_tags):
		self.pos_tags = pos_tags
		for index,elem in enumerate(self.tokens):
			if(elem=="\n"):
				self.pos_tags[index] = ""
			if(elem=="EOS"):
				self.pos_tags[index] = "EOS"

	def set_lemas(self,lemas):
		self.lemas = lemas
		for index,elem in enumerate(self.tokens):
			if(elem=="\n"):
				self.lemas[index] = ""
			if(elem=="EOS"):
				self.lemas[index] = "EOS"

	def set_entities(self,entities):
		self.entities = entities
		for index,elem in enumerate(self.tokens):
			if(elem=="\n"):
				self.entities[index] = ""
			if(elem=="EOS"):
				self.entities[index] = "EOS"

	def set_np_tags(self,np_tags):
		self.np_tags = np_tags
		for index,elem in enumerate(self.tokens):
			if(elem=="\n"):
				self.np_tags[index] = ""
			if(elem=="EOS"):
				self.np_tags[index] = "EOS"

	def print_text_order(self,lista):
		for elem in lista:
			print(elem)

	def print_conll(self):
		current_line = ""
		for index in range(len(self.tokens)):
			if(self.tokens!=[]):
				current_line += self.tokens[index]
			if(self.pos_tags!=[]):
				current_line += " " + self.pos_tags[index]
			if(self.lemas!=[]):
				current_line += " " + self.lemas[index]
			if(self.entities!=[]):
				current_line += " " + self.entities[index]
			if(self.np_tags!=[]):
				current_line += " " + self.np_tags[index]
			print(current_line)
			current_line=""


class dictionary_entry:
    def __init__(self):
        self.inflected_form = None
        self.lemma = None
        self.part_of_speech = None
        self.subcategory = None
        self.morph_attributes = None


    def __init__(self,inflected_form=None,lemma=None,part_of_speech=None,subcategory=None,morph_attributes=None):
        self.inflected_form = inflected_form
        self.lemma = lemma
        self.part_of_speech = part_of_speech
        self.subcategory = subcategory
        self.morph_attributes = morph_attributes

    def print_entry(self):
        print("Inflected form : " + self.inflected_form)
        print("Lemma form : " + self.lemma)
        print("Part of speech form : " + self.part_of_speech)
        print("Subcategory form : " + self.subcategory)
        print("Morph attributes form : " + self.morph_attributes)

#int hashCode function
#bollean equals


# -*- coding: utf-8 -*-
"""
Created on Wed Nov 21 13:03:21 2018

@author: fabio
"""
import warnings
warnings.filterwarnings("ignore")
from collections import Counter
from itertools import chain
from sklearn.model_selection import *
from sklearn.metrics import *
from sklearn.preprocessing import LabelBinarizer,LabelEncoder
from sklearn.decomposition import PCA
from sklearn.metrics.pairwise import cosine_similarity
from scipy import spatial
import re
import pandas as pd
from pandas import *
import numpy as np
import os
import itertools
import pickle
#import matplotlib.pyplot as plt
from sklearn_crfsuite import CRF
from sklearn_crfsuite import metrics
from sklearn_crfsuite import scorers
from pathlib import Path
start_path = Path(__file__).parent.absolute()

def print_transitions(trans_features):
    string = ""
    for (label_from, label_to), weight in trans_features:
        string += ("%-6s -> %-7s %0.6f" % (label_from, label_to, weight))
        string+="\n"
    return string
def print_state_features(state_features):
    string = ""
    for (attr, label), weight in state_features:
        string+=("%0.6f %-8s %s" % (weight, label, attr))
        string+="\n"
    return string

class GenerateFeatures():
    def __init__(self):
        self.features={}
    def generateMorphologyFeatures(self,word,tag):
        self.features[tag+'contains_punct']=str(0)
        self.features[tag+'word_shape']=''
        for char in word:
            if str(char).isalpha():
                if str(char).islower():
                    self.features[tag+'word_shape']+='a'
                else:
                    self.features[tag+'word_shape']+='A'
            elif str(char).isdigit():
                self.features[tag+'word_shape']+='#'
            else:
                self.features[tag+'word_shape']+='-'
                self.features[tag+'contains_punct']=str(1)
        self.features[tag+'ends_in_s']=str(word[-1:]=='s')
        self.features[tag+'ends_in_a']=str(word[-1:]=='a')
        self.features[tag+'ascii']=str(len(word)==len(word.encode()))
        self.features[tag+'is_all_lowercase']=str(word.lower()==word and word.isdigit()==False)
        self.features[tag+'is_first_uppercase']=str(word.isupper())
        self.features[tag+'is_complete_uppercase']=str(word.upper()==word and word.isdigit()==False)
        self.features[tag+'is_numeric']=str(word.isdigit())
        self.features[tag+'is_alpha']=str(word.isalpha())
        self.features[tag+'is_alphanumeric']=str(bool((re.match('^(?=.*[0-9]$)(?=.*[a-zA-Z])',word))))
        self.features[tag+'length']=str(word)
        prefix_suffix_window=5
        for i in range(1,prefix_suffix_window+1):
            self.features[tag+'prefix_'+str(i)]=str(word[:i])
            self.features[tag+'suffix_'+str(i)]=str(word[-i:])
    def wordcontextFeatures(self,word,pos,lemma,tag):
        self.features[tag+'word']=str(word)
        self.features[tag+'pos']=pos
        self.features[tag+'lemma']=str(lemma)
            
def features(document,pos_list,lemma_list,index,useful_features):
    word=document[index]
    pos=pos_list[index]
    lemma=lemma_list[index]
    if index-1>0:
        prev_prev_word=document[index-2]
        prev_prev_pos=pos_list[index-2]
        prev_prev_lemma=lemma_list[index-2]
    else:
        prev_prev_word=''
        prev_prev_pos=''
        prev_prev_lemma=''
    if index>0:
        prev_word=document[index-1]
        prev_pos=pos_list[index-1]
        prev_lemma=lemma_list[index-1]
    else:
        prev_word=''
        prev_pos=''
        prev_lemma=''
    if index<len(document)-1:
        next_word=document[index+1]
        next_pos=pos_list[index+1]
        next_lemma=lemma_list[index+1]
    else:
        next_word=''
        next_pos=''
        next_lemma=''
    if index<len(document)-2:
        next_next_word=document[index+2]
        next_next_pos=pos_list[index+2]
        next_next_lemma=lemma_list[index+2]
    else:
        next_next_word=''
        next_next_pos=''
        next_next_lemma=''
    gen_features=GenerateFeatures()
    if useful_features[0]:
        gen_features.generateMorphologyFeatures(word,'')
        gen_features.generateMorphologyFeatures(prev_word,'prev_')
        gen_features.generateMorphologyFeatures(next_word,'next_')
        gen_features.generateMorphologyFeatures(prev_prev_word,'prev_prev_')
        gen_features.generateMorphologyFeatures(next_next_word,'next_next_')
    if useful_features[1]:
        gen_features.wordcontextFeatures(word,pos,lemma,'')
        gen_features.wordcontextFeatures(prev_word,prev_pos,prev_lemma,'prev_')
        gen_features.wordcontextFeatures(next_word,next_pos,next_lemma,'next_')
        gen_features.wordcontextFeatures(prev_prev_word,prev_prev_pos,prev_prev_lemma,'prev_prev_')
        gen_features.wordcontextFeatures(next_next_word,next_next_pos,next_next_lemma,'next_next_')
    return gen_features.features

def untag(document,option):
    #Passar de colunas para tuples
    if option!='predict':
        #return [str(word) for word,pos,tag in document],[pos for word,pos,tag in document],['' for word,pos,tag in document]
        return [str(word) for word,pos,lemma,tag in document],[pos for word,pos,lemma,tag in document],[lemma for word,pos,lemma,tag in document]
    else:
        #return [str(word) for word,pos in document],[pos for word,pos in document],['' for word,pos in document]
        return [str(word) for word,pos,lemma in document],[pos for word,pos,lemma in document],[lemma for word,pos,lemma in document]

def prepareData(tagged_documents,option,useful_features):
    #Construir o dataset a partir dos ficheiros -> Gerar features para usar no CRF
    X,y=[],[]
    for document in tagged_documents:
        words,postags,lemmas =untag(document,option)
        X.append([features(words,postags,lemmas,index,useful_features) for index in range(len(document))])
        if option!='predict':
            y.append([tag for word,pos,lemmas,tag in document])
    return X,y

def fromListToTuple(data):
    tuple_data=[]
    for l in data:
        tuple_data.append(tuple(l))
    return tuple_data

def fromTuplesToList(data,y):
    dataset=[]
    for i in range(len(data)):
        dataset.append([])
        for j in range(len(data[i])):
            list_j=list(data[i][j])
            list_j.append(y[i][j])
            dataset[i].append(list_j)
    return dataset

def save_model(nome,modelo):
    with open(nome,'wb') as f:
        pickle.dump(modelo,f,protocol=pickle.HIGHEST_PROTOCOL)

def load_model(nome):
    with open(nome,'rb') as f:
         result = pickle.load(f)
    return result

def run_crf(data,load_file=start_path.joinpath("harem.pickle")):
    useful_features=[True,True]
    X_test=fromListToTuple(data)
    X_teste,y_teste=prepareData([X_test],'predict',useful_features)
    crf = load_model(load_file)
    y_pred=crf.predict(X_teste)
    resultados = []
    for index,elem in enumerate(y_pred[0]):
        resultados.append(str(y_pred[0][index]))
    return resultados

def test_crf(train_file,test_file,model_name=""):
    valores = []
    data=pandas.read_csv(train_file,sep="\t",header=None)
    X_dataset=fromListToTuple(data.iloc[:,[0,1,2,3]].values)
    useful_features=[True,True]
    X_train,y_train=prepareData([X_dataset],'train',useful_features)
    data2=pandas.read_csv(test_file,sep="\t",header=None)
    X_test=fromListToTuple(data2.iloc[:,[0,1,2]].values)
    X_teste,y_teste=prepareData([X_test],'predict',useful_features)
    crf = CRF(
            algorithm='lbfgs',
            c1=0.0625,
            c2=0.5,
            max_iterations=100,
            all_possible_transitions=False,
            all_possible_states=True,
            verbose=True
        )
    crf.fit(X_train, y_train)
    if(model_name!=""):
        save_model(model_name + ".pickle",crf)
    useful_features=[True,True]
    data2=pandas.read_csv(test_file,sep="\t",header=None)
    X_test=fromListToTuple(data2.iloc[:,[0,1,2]].values)
    X_teste,y_teste=prepareData([X_test],'predict',useful_features)
    y_pred=crf.predict(X_teste)
    resultados = []
    for index,elem in enumerate(y_pred[0]):
        resultados.append(str(y_pred[0][index]))
    return resultados




import xmltodict
import re

class superlative_normalizer:
	def __init__(self):

		self.declesion_exceptions=[]
		self.declesion_targets = []
		self.declesion_tags = []
		self.declesion_replacement = []
		self.declesion_prefix = []
		self.declesion_re = []

	def compile_rules(self):
		lista = [None] * len(self.declesion_targets)
		for index,elem in enumerate(self.declesion_targets):
			li = re.compile(self.declesion_prefix+elem)
			lista[index] = li
		self.declesion_re=lista


	def superlative_normalizer_load(self,file_name):
		#self.declesion_replacement = declesions
		#self.declesion_replacement.sort
		with open(file_name) as file:
			doc = xmltodict.parse( file.read())
			result = (doc["superlativedeclensions"]["replacement"])
			'''
			Get the number of rules and set the size of the class variables
			'''
			genderdeclensions_number_of_rules = len(result)
			self.declesion_exceptions=[None]*genderdeclensions_number_of_rules
			self.declesion_targets = [None]*genderdeclensions_number_of_rules
			self.declesion_tags = [None]*genderdeclensions_number_of_rules
			self.declesion_replacement = [None]*genderdeclensions_number_of_rules
			'''
			Check if the rule has any of the elements 
			'''
			for index,elem in enumerate(result):
				if '@target' in elem.keys():
					self.declesion_targets[index] = elem["@target"]
				else:
					self.declesion_targets[index] = ""
				if '@exceptions' in elem.keys():
					self.declesion_exceptions[index] = elem["@exceptions"]
				else:
					self.declesion_exceptions[index] = ""
				if '@tag' in elem.keys():
					self.declesion_tags[index] = elem["@tag"]
				else:
					self.declesion_tags[index] = ""
				if '#text' in elem.keys():
					self.declesion_replacement[index] = elem["#text"]
				else:
					self.declesion_replacement[index] = ""
					
			self.declesion_prefix = (doc["superlativedeclensions"]["prefix"])


	def print_superlative_normalizer(self):
		for i in range(len(self.declesion_targets)):
			print(i)
			print("---------\n")
			print("Target: " + self.declesion_targets[i])
			print("Tags: " + self.declesion_tags[i])
			print("Exceptions: " + self.declesion_exceptions[i])
			print("Replacement: " + self.declesion_replacement[i])

	def normalize_superlative(self,token, tag):
		#choose biggest rule
		bigget_rule_size = 0
		#lemmatized_word = token.lower()
		lemmatized_word = ""
		'''
		Token to lowercase to match the rules
		'''
		normalization = token.lower()
		'''
		Check for every rule it the element matches the target,
		if the tags match and if the element is not an exception

		'''
		for index,elem in enumerate(self.declesion_targets):
			word = normalization[-len(self.declesion_targets[index]):]
			#print("-----"+elem+"----"+word)
			if( self.declesion_re[index].fullmatch(normalization)
				and tag.lower() in self.declesion_tags[index].split("|")
				and not (normalization in self.declesion_exceptions[index].split("|"))):
					tamanho = len(self.declesion_targets[index])
					if(tamanho>bigget_rule_size):
						lemmatized_word = normalization[:(len(normalization) - len(self.declesion_targets[index]))] + self.declesion_replacement[index]
						bigget_rule_size=tamanho
						return(lemmatized_word)
		return(lemmatized_word)





class word_ranking:
    def __init__(self):
        self.wordMap = {}

    def load(self,input_file):
        rank = 0
        frequency = 0
        word = 0
        lineNumber = 0
        with open(input_file,"r",encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                if(len(line)>0 and not (line[0]=="#")):
                    if("\t" in line and (line.find("\t")+1<len(line))):
                        word = line[line.find("\t")+1:].replace("="," ")
                        frequency = line[0:line.find("\t")]
                        rank += 1
                        new_entry = word_list_entry(word,frequency,rank)
                        #new_entry.print_word_list_entry()
                        self.wordMap[word] = new_entry
                    else:
                        break
        #print(self.get_frequency("-"))
        #print(self.retrieve_top_word(["minsitro","mielóide"]))
        #print(self.retrieve_top_word(["mielóide","minsitro"]))

    def get_frequency(self , word):
        if( word in self.wordMap):
            return self.wordMap[word].frequency
        else:
            return -1

    def get_rank(self , word):
        if(word in self.wordMap):
            return self.wordMap[word].rank
        else:
            return -1

    def rank(self,words,limit=0):

        ranked_list = []
        for word in words:
            if(not(self.get_rank(word)==-1)):
                new_entry = word_list_entry(word,self.get_frequency(word),self.get_rank(word))
                ranked_list.append(new_entry)
            else:
                new_entry = word_list_entry(word, 9999999999, 9999999999)
                ranked_list.append(new_entry)
        ranked_list.sort(key=lambda x: x.rank)
        if(limit==0):
            limit = len(ranked_list)
        ranked_words=[None]*len(ranked_list)
        ranked_list = ranked_list[0:limit+1]
        for index,elem in enumerate(ranked_list):
            ranked_words[index]=elem.word
        return ranked_words

        
    def retrieve_top_word(self,words):
        top_word = None
        if(len(words)>0):
            result =self.rank(words,1)
            if(len(result)>0):
                top_word=result[0]
            else:
                top_word = None
        return top_word
        
####################    
class word_list_entry:
    def __init__(self,word=None,frequency=0,rank=0):
        self.word = word
        self.frequency = frequency
        self.rank = rank

    def compare_to(self,other):
        if(self.frequency < other.frequency):
            return 1
        elif (self.frequency > other.frequency):
            return -1
        else:
            if(self.rank < other.rank):
                return -1
            elif (self.rank > other.rank):
                return 1
            else:
                return 0
        
    def print_word_list_entry(self):
        to_string = str(self.rank) + "\t" + str(self.frequency) + "\t" + str(self.word)
        print(to_string)
        return(to_string)

if __name__ == "__main__":
    novo_rank = word_ranking()
    novo_rank.load("acdc/lemas.total.txt")


from setuptools import setup
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'NLPyPort',         # How you named your package folder (MyLib)
  packages = ['.',
  'NLPyPort/',
  "NLPyPort/LemPyPort/normalization",
"NLPyPort/LemPyPort",
"NLPyPort/LemPyPort/dictionary",
"NLPyPort/LemPyPort/rank",
"NLPyPort/LemPyPort/resources/replacements",
"NLPyPort/LemPyPort/resources/acdc",
"NLPyPort/LemPyPort/resources/config",
"NLPyPort/LemPyPort/resources/label",
"NLPyPort/LemPyPort/resources/dictionaries",
"NLPyPort/TagPyPort",
"NLPyPort/TokPyPort",
"NLPyPort/TagPyPort/resources/config",
"NLPyPort/TagPyPort/resources/PoSModels",
"NLPyPort/TokPyPort/config",
"NLPyPort/TokPyPort/resources/config",
"NLPyPort/TokPyPort/resources/replacements",
"NLPyPort/TokPyPort",
"NLPyPort/CRF",
"NLPyPort/config",
"NLPyPort/CRF/trainedModels"
  ], 
  package_data={
"NLPyPort/LemPyPort/resources/replacements":['*'],
"NLPyPort/LemPyPort/resources/acdc":['*'],
"NLPyPort/LemPyPort/resources/config":['*'],
"NLPyPort/LemPyPort/resources/label":['*'],
"NLPyPort/LemPyPort/resources/dictionaries":['*'],
"NLPyPort":['*'],
"NLPyPort/config":['*'],
"NLPyPort/config/":['*'],
"NLPyPort/TagPyPort/resources/config":['*'],
"NLPyPort/TagPyPort/resources/PoSModels":['*'],
"NLPyPort/TokPyPort/resources/replacements":['*'],
"NLPyPort/TokPyPort/resources/config":['*'],
"NLPyPort/TokPyPort/config":['*'],
"NLPyPort/CRF":['*'],
"NLPyPort/CRF/.vscode":['*'],
"NLPyPort/CRF/trainedModels":['*'],
},  # Chose the same as "name"
  version = '2.2.5',      # Start with a small number and increase it with every change you make
  license='cc0-1.0',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Python NLP for Portuguese',   # Give a short description about your library
  author = 'Joao Ferreira',                   # Type in your name
  author_email = 'jdhtml5@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/jdportugal/NLPyPort',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/jdportugal/NLPyPort',    # I explain this later on
  keywords = ['NLP', 'PORTUGUESE', 'PYTHON'],   # Keywords that define your package best
  long_description=long_description,
  long_description_content_type='text/markdown',
  install_requires=[            # I get to this in a second
         "nltk",
"numpy",
"pandas",
"python-crfsuite",
"python-dateutil",
"pytz",
"scikit-learn",
"scipy",
"singledispatch",
"six",
"sklearn",
"sklearn-crfsuite",
"tabulate",
"tqdm",
"xmltodict",

      ],
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)


# -*- coding: utf-8 -*-
"""
@author: João Ferreira
"""
import nltk.data
import os
from nltk.corpus import floresta
from nltk.tokenize import sent_tokenize,word_tokenize
from nltk.corpus.reader import TaggedCorpusReader
from nltk.tokenize import LineTokenizer
from nltk.corpus import treebank
from nltk.metrics import accuracy
from nltk.corpus import machado
import pickle
import nltk
import time
import xmltodict
from pathlib import Path
from NLPyPort.text import *
from NLPyPort.TokPyPort import *
from NLPyPort.LemPyPort import *
from NLPyPort.TagPyPort import *
from NLPyPort.CRF import *

start_path = Path(__file__).parent.absolute()
global_porperties_file = start_path.joinpath("config/global.properties")

lexical_conversions="PRP:PREP;PRON:PRO;IN:INTERJ;ART:DET;"
floresta.tagged_words(tagset = "pt-bosque")
TokPort_config_file = ""
TagPort_config_file = ""
LemPort_config_file = ""
debug = False

def debug_print(string):
	global debug
	if(debug==True):
		print(string)

def load_congif_to_list():
	config_list=[]
	TokPort_config_file,TagPort_config_file,LemPort_config_file = load_and_return_config()
	config_list.append(TokPort_config_file)
	config_list.append(TagPort_config_file)
	config_list.append(LemPort_config_file)
	for elem in load_and_return_lemmatizer(LemPort_config_file):
		config_list.append(elem)
	
	return config_list


def unload_config_from_list(config_list):
	TokPort_config_file = config_list[0]
	TagPort_config_file = config_list[1]
	LemPort_config_file = config_list[2]
	adverb_norm = config_list[3]
	number_norm = config_list[4]
	superlative_norm = config_list[5]
	augmentative_norm = config_list[6]
	diminutive_norm = config_list[7]
	gender_norm = config_list[8]
	gender_name_norm = config_list[9]
	verb_norm = config_list[10]
	ranking = config_list[11]
	novo_dict = config_list[12]
	return TokPort_config_file,TagPort_config_file,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict



def load_config(config_file=global_porperties_file):
	global TokPort_config_file
	global TagPort_config_file
	global LemPort_config_file
	with open (config_file,'r') as f:
		for line in f:
			if(line[0]!="#"):
				if(line.split("=")[0]=="TokPort_config_file"):
					TokPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))
				elif(line.split("=")[0]=="TagPort_config_file"):
					TagPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))
				elif(line.split("=")[0]=="LemPort_config_file"):
					LemPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))

def load_and_return_config(config_file=global_porperties_file):
	with open (config_file,'r') as f:
		for line in f:
			if(line[0]!="#"):
				if(line.split("=")[0]=="TokPort_config_file"):
					TokPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))
				elif(line.split("=")[0]=="TagPort_config_file"):
					TagPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))
				elif(line.split("=")[0]=="LemPort_config_file"):
					LemPort_config_file = start_path.joinpath(line.split("=")[1].strip("\n"))
	return TokPort_config_file,TagPort_config_file,LemPort_config_file


def load_and_return_lemmatizer(LemPort_config_file):
	adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict = nlpyport_lematizer_loader(LemPort_config_file)
	return [adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict]



# """word tokenizer"""
def tokenize(fileinput,tok_config=""):
	if(tok_config==""):
		tok_config = TokPort_config_file
	return nlpyport_tokenizer(fileinput,tok_config)

def tokenize_from_string(stringinput,tok_config=""):
	if(tok_config==""):
		tok_config = TokPort_config_file
	return nlpyport_tokenize_from_string(stringinput,tok_config)

def tag(tokens,tag_config=""):
	if(tag_config==""):
		tag_config = TagPort_config_file
	return nlpyport_pos(tokens,tag_config)


def lematizador_preloaded(tokens,tags):
	global LemPort_config_file
	mesmas = 0
	alteradas = 0
	resultado = nlpyport_lematizer(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
	return resultado

def lematizador_normal(tokens,tags):
	global LemPort_config_file
	mesmas = 0
	alteradas = 0
	resultado = nlpyport_lematizer(tokens,tags,LemPort_config_file)
	return resultado

def load_manual(file):
	tokens = []
	tags = []
	f =  open(file,'r')
	alteradas = 0
	mesmas = 0
	for line in f:
		res = line.split(" ")
		if(len(res)>1):
			tokens.append(res[0])
			tags.append(res[1].split('\n')[0])
	return tokens,tags

def write_lemmas_only_text(lem,file="testes.txt"):
	for elem in lem:
		with open(file,'a') as f:
			if(elem == '\n'):
				f.write('\n')
			else:
				f.write(str(elem)+" ")

def write_simple_connl(tokens,tags,lems,ents,nps,file=""):
	linhas = 0
	if(file != ""):
		for index in range(len(tokens)):
			with open(file,'a') as f:
				if(tokens[index] == "\n"):
					f.write("\n")
					linhas = 0
				else:
					linhas += 1
					f.write(str(linhas) + ", " + str(tokens[index] + ", " +str(lems[index]) + ", " + str(tags[index])+", "+str(ents[index]) + ", " + str(nps[index])+"\n"))
	else:
		for index in range(len(tokens)):
			if(tokens[index] == "\n"):
				print("\n")
				linhas = 0
			else:
				linhas += 1
				print(str(linhas) + ", " + str(tokens[index]) + ", " +str(lems[index]) + ", " + str(tags[index]) + ", " + str(nps[index]))

def lem_file(out,token,tag):
	lem = []
	ent = []
	lem = lematizador_normal(token,tag)
	with open(out,"wb") as f:
		for i in range(len(token)):
			line = token[i] +"\t" +tag[i] + "\t" + lem[i]  + "\n"
			f.write((line).encode('utf8'))
	return lem

def join_data(tokens,tags,lem):
	data = []
	for i in range(len(tokens)):
		dados = []
		dados.append(tokens[i]) 
		dados.append(tags[i]) 
		dados.append(lem[i]) 
		data.append(dados)
	return data

def full_pipe(input_file,out_file=""):

	#load the pipeline configurations
	load_config()
	
	#############
	#Tokenize
	#############
	
	tokens = tokenize(input_file)


	#############
	#Replace the privious line by these lines if you want to tokenize
	#from a string or from a list of strings rather than a file
	#############
	#string_input = input_file  # this should be either a list or a string
	#tokens = tokenize_from_string(string_input)

	
	#############
	#Pos
	#############
	tags,result_tags = tag(tokens)
	
	#### optional - use a file with tokens and tags as input
	#tokens,tags = load_manual(input_file)
	#tokens,tags = load_manual("TokPyPort/conll_tagged_text_all.txt")

	#############
	#Lemmatizer
	#############
	lemas = lematizador_normal(tokens,tags)
	#Re-write the file with the lemas
	#write_lemmas_only_text(lemas,"File.txt")

	###################
	#Entity recognizer
	###################

	entidades = []
	joined_data = join_data(tokens,tags,lemas)
	trained_model = start_path.joinpath("CRF/trainedModels/harem.pickle")
	entidades = run_crf(joined_data,trained_model)


	###################
	#NP chunking
	###################

	np_tags = []
	joined_data = join_data(tokens,tags,lemas)
	np_model = start_path.joinpath("CRF/NP_Final.pickle")
	#Alternative model macro optimized
	#np_model = "CRF/NP_Final_Macro.pickle"

	np_tags = run_crf(joined_data,np_model)
	


	write_simple_connl(tokens,tags,lemas,entidades,np_tags,out_file)

	return tokens,tags,lemas,entidades,np_tags

def new_full_pipe(input_file,options=[],config_list=[]):
	print(options)
	text = Text()
	if(options==[]):
		options = {
			"tokenizer" : True,
			"pos_tagger" : True,
			"lemmatizer" : True,
			"entity_recognition" : True,
			"np_chunking" : True,
			"pre_load" : False,
			"string_or_array" : False
		}

	###########################
	#Acrescentar a forma de fazer so load de cada parte da pipeline?
	#Acrescentar excepção se ficheiro nao exisitr
	###########################

	if("pre-load" in options):
		if(options["pre-load"]==True):
			if(config_list!=[]):
				TokPort_config_file,TagPort_config_file,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict = unload_config_from_list(config_list)
				debug_print("Pipeline using pre-loaded models.")
			else:
				print("To use the pre-load option a list of loaded configurations must be passed.")
				return 0
		elif(options["pre-load"]==False):
			load_config()
			debug_print("Pre-load set to false, configurations loaded")
	else:
		options["pre-load"] = False
		load_config()
		debug_print("No pre-load choosen, pre-load set to default (false), configurations loaded")

	if(not("pos_tagger" in options) and not("lemmatizer" in options) and not("entity_recognition" in options) and not("np_chunking") in options):
		print("No text processing option given, performing the full pipeline processement.")
		options["tokenizer"] = True
		options["pos_tagger"] = True
		options["lemmatizer"] = True
		options["entity_recognition"] = True
		options["np_chunking"] = True
		options["pre-load"] = False

	if("tokenizer" in options):
		if(options["tokenizer"]==True):
			debug_print("Tokenizer is True")
			if("string_or_array" in options):
				if(options["string_or_array"]==False):
					if(options["pre-load"]==False):
						tokens = tokenize(input_file)
						text.set_tokens(tokens)
						debug_print("Pre-load option set to False, tokens gotten")
					else:
						tokens = tokenize(input_file,TokPort_config_file)
						text.set_tokens(tokens)
						debug_print("Pre-load option set to True, Tokens gotten")
				else:
					if(options["pre-load"]==False):
						debug_print("Pre-load is False")
						tokens = tokenize_from_string(input_file)
						text.set_tokens(tokens)
					else:
						debug_print("Pre-load is True")
						tokens = tokenize_from_string(input_file,TokPort_config_file)
						text.set_tokens(tokens)
			else:
				if(options["pre-load"]==False):
						tokens = tokenize(input_file)
						text.set_tokens(tokens)
						debug_print("Pre-load option set to False, tokens gotten")
				else:
					tokens = tokenize(input_file,TokPort_config_file)
					text.set_tokens(tokens)
					debug_print("Pre-load option set to True, Tokens gotten")
	else:
		##########################
		#Alterar, no caso se não haver tokenizer necessário passar uma list
		##########################
		if("string_or_array" in options):
			if(options["string_or_array"]==False):
				print("If no tokenization is needed please give the input as an array os tring and set 'string_or_array' to False")
				return 0
		#If no data given assume its a file
			else:
				if(options["pre-load"]==False not in options):
						tokens = tokenize_from_string(input_file)
						text.set_tokens(tokens)
						debug_print("Pre-load option set to False, tokens gotten")
				else:
					tokens = tokenize_from_string(input_file,TokPort_config_file)
					text.set_tokens(tokens)
	if("pos_tagger" in options):
		if(options["pos_tagger"] == True):
				debug_print("Pos_tagger is True")
				if(options["pre-load"]==False):
					tags,result_tags = tag(text.tokens)
					text.set_pos_tags(tags)
					debug_print("Pre-load option set to False, Pos-tags gotten")
				else:
					tags,result_tags = tag(text.tokens,TagPort_config_file)
					text.set_pos_tags(tags)
					debug_print("Pre-load option set to True, Pos-tags gotten")
		else:
			make_pos = False
			if( "lemmatizer" in options):
				if(options["lemmatizer"]==True):
					if(options["pre-load"]==False):
						tags,result_tags = tag(text.tokens)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					else:
						tags,result_tags = tag(text.tokens,TagPort_config_file)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					make_pos = True
			if( "entity_recognition" in options and make_pos==False):
				if(options["entity_recognition"]==True):
					if(options["pre-load"]==False):
						tags,result_tags = tag(text.tokens)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					else:
						tags,result_tags = tag(text.tokens,TagPort_config_file)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					make_pos = True
			if( "np_chunking" in options and make_pos==False):
				if(options["np_chunking"]==True):
					if(options["pre-load"]==False):
						tags,result_tags = tag(text.tokens)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					else:
						tags,result_tags = tag(text.tokens,TagPort_config_file)
						text.set_pos_tags(tags)
						debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
					make_pos = True
	else:
		make_pos = False
		if( "lemmatizer" in options):
			if(options["lemmatizer"]==True):
				if(options["pre-load"]==False):
					tags,result_tags = tag(text.tokens)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				else:
					tags,result_tags = tag(text.tokens,TagPort_config_file)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				make_pos = True
		if( "entity_recognition" in options and make_pos==False):
			if(options["entity_recognition"]==True):
				if(options["pre-load"]==False):
					tags,result_tags = tag(text.tokens)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				else:
					tags,result_tags = tag(text.tokens,TagPort_config_file)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				make_pos = True
		if( "np_chunking" in options and make_pos==False):
			if(options["np_chunking"]==True):
				if(options["pre-load"]==False):
					tags,result_tags = tag(text.tokens)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				else:
					tags,result_tags = tag(text.tokens,TagPort_config_file)
					text.set_pos_tags(tags)
					debug_print("Pos tagging was made since it's needed for other elements of the pipeline.")
				make_pos = True

	if("lemmatizer" in options):
		if(options["lemmatizer"] == True):
				debug_print("Lemmatizer is True")
				if(options["pre-load"]==False):
					lemas = lematizador_normal(tokens,tags)
					text.set_lemas(lemas)
					debug_print("Pre-load option set to False, Lemmas gotten")
				else:
					lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
					text.set_lemas(lemas)
					debug_print("Pre-load option set to True, Lemmas gotten")
		else:
			make_lem = False
			if( "entity_recognition" in options and make_lem==False):
					if(options["entity_recognition"]==True):
						if(options["pre-load"]==False):
							lemas = lematizador_normal(tokens,tags)
							text.set_lemas(lemas)
							debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
						else:
							lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
							text.set_lemas(lemas)
							debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
						make_lem = True
			if( "np_chunking" in options and make_lem==False):
				if(options["np_chunking"]==True):
					if(options["pre-load"]==False):
						lemas = lematizador_normal(tokens,tags)
						text.set_lemas(lemas)
						debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
					else:
						lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
						text.set_lemas(lemas)
						debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
					make_lem = True
	else:
		make_lem = False
		if( "entity_recognition" in options and make_lem==False):
				if(options["entity_recognition"]==True):
					if(options["pre-load"]==False):
						lemas = lematizador_normal(tokens,tags)
						text.set_lemas(lemas)
						debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
					else:
						lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
						text.set_lemas(lemas)
						debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
					make_lem = True
		if( "np_chunking" in options and make_lem==False):
			if(options["np_chunking"]==True):
				if(options["pre-load"]==False):
					lemas = lematizador_normal(tokens,tags)
					text.set_lemas(lemas)
					debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
				else:
					lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
					text.set_lemas(lemas)
					debug_print("Lemmatization was made since it's needed for other elements of the pipeline.")
				make_lem = True

	if("entity_recognition" in options):
		if(options["entity_recognition"] == True):
				debug_print("Entity_recognition is True")
				joined_data = join_data(text.tokens,text.pos_tags,text.lemas)
				trained_model = start_path.joinpath("CRF/trainedModels/harem.pickle")
				entidades = run_crf(joined_data,trained_model)
				text.set_entities(entidades)
				debug_print("Entities gotten")

	if("np_chunking" in options):
		if(options["np_chunking"] == True):
				debug_print("Np_chunking is True")
				np_tags = []
				joined_data = join_data(tokens,tags,lemas)
				np_model = start_path.joinpath("CRF/NP_Final.pickle")
				np_tags = run_crf(joined_data,np_model)
				text.set_np_tags(np_tags)
				debug_print("NP-Chunks gotten")
	
	return text




def full_pipe_preload(input_file,config_list,out_file=""):
	TokPort_config_file,TagPort_config_file,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict = unload_config_from_list(config_list)
	
	
	#############
	#Tokenize
	#############

	tokens = tokenize(input_file,TokPort_config_file)

	#############
	#Replace the privious line by these lines if you want to tokenize
	#from a string or from a list of strings rather than a file
	#############
	#string_input = input_file  # this should be either a list or a string
	#tokens = tokenize_from_string(string_input,TokPort_config_file)

	
	#############
	#Pos
	#############
	tags,result_tags = tag(tokens,TagPort_config_file)

	#############
	#Lemmatizer
	#############
	lemas = nlpyport_lematizer_preload(tokens,tags,LemPort_config_file,adverb_norm,number_norm,superlative_norm,augmentative_norm,diminutive_norm,gender_norm,gender_name_norm,verb_norm,ranking,novo_dict)
	#Re-write the file with the lemas only
	#write_lemmas_only_text(lemas,"File.txt")

	###################
	#Entity recognition
	###################
	entidades = []
	joined_data = join_data(tokens,tags,lemas)
	trained_model = start_path.joinpath("/CRF/trainedModels/trainedModels/harem.pickle")
	entidades = run_crf(joined_data,trained_model)

	np_tags = []
	joined_data = join_data(tokens,tags,lemas)
	np_model = start_path.joinpath("/CRF/trainedModels/NP_Final.pickle")
	#Alternative model macro optimized
	#np_model = "CRF/NP_Final_Macro.pickle"
	np_tags = run_crf(joined_data,np_model)
	

	write_simple_connl(tokens,tags,lemas,entidades,np_tags,out_file)
	
	return tokens,tags,lemas,entidades,np_tags

####################
#Alterar NER e NP para nao devolver nada em casos em que esta vazio
#Colocar togas tags NP em maiúscula
####################


if __name__ == "__main__":
	input_file = "SampleInput/Sample.txt"
	out_file = "SampleOut.txt"
	out_file_2 = "SampleOut2.txt"
	frase_input = "Sou uma frase de input!"
	array_input = ["Sou uma frase de input!","Mas faço parte de um array"]
	#################################
	#Run the full pipeline for a file
	#################################
	options = {"string_or_array":True,"pre-load":True,"tokenizer":True,"lemmatizer":True,"entity_recognition":True,"np_chunking":True}
	config_list = load_congif_to_list()
	#text = new_full_pipe("SampleInput/Sample.txt",{"string_or_array":False,"pre-load":True,"tokenizer":False,"pos_tagger":True,"lemmatizer":True,"entity_recognition":False,"np_chunking":True},config_list)
	text = new_full_pipe(input_file=array_input,options={"string_or_array":True})
	if(text!=0):
		text.print_connl()
	#print(text.tokens)


	#######################################################################
	#Optional pre-load
	#######################################################################
	####
	#Pre-load the configurations
	####
	#config_list = load_congif_to_list()

	####
	#Run the pipeline
	####
	#full_pipe_preload(input_file,config_list,out_file_2)



# -*- coding: utf-8 -*-
"""
Created on Wed Nov 21 13:03:21 2018

@author: fabio
"""
import warnings
warnings.filterwarnings("ignore")
from collections import Counter
from itertools import chain
from sklearn.model_selection import *
from sklearn.metrics import *
from sklearn.preprocessing import LabelBinarizer,LabelEncoder
from sklearn.decomposition import PCA
from sklearn.metrics.pairwise import cosine_similarity
from scipy import spatial
import re
import pandas as pd
from pandas import *
import numpy as np
import os
import itertools
import pickle
#import matplotlib.pyplot as plt
from sklearn_crfsuite import CRF
from sklearn_crfsuite import metrics
from sklearn_crfsuite import scorers
import random
from sklearn.model_selection import train_test_split
random.seed(1)

def print_transitions(trans_features):
	string = ""
	for (label_from, label_to), weight in trans_features:
		string += ("%-6s -> %-7s %0.6f" % (label_from, label_to, weight))
		string+="\n"
	return string
def print_state_features(state_features):
	string = ""
	for (attr, label), weight in state_features:
		string+=("%0.6f %-8s %s" % (weight, label, attr))
		string+="\n"
	return string

class GenerateFeatures():
	def __init__(self):
		self.features={}
	def generateMorphologyFeatures(self,word,tag):
		self.features[tag+'contains_punct']=str(0)
		self.features[tag+'word_shape']=''
		for char in word:
			if str(char).isalpha():
				if str(char).islower():
					self.features[tag+'word_shape']+='a'
				else:
					self.features[tag+'word_shape']+='A'
			elif str(char).isdigit():
				self.features[tag+'word_shape']+='#'
			else:
				self.features[tag+'word_shape']+='-'
				self.features[tag+'contains_punct']=str(1)
		self.features[tag+'ends_in_s']=str(word[-1:]=='s')
		self.features[tag+'ends_in_a']=str(word[-1:]=='a')
		self.features[tag+'ascii']=str(len(word)==len(word.encode()))
		self.features[tag+'is_all_lowercase']=str(word.lower()==word and word.isdigit()==False)
		self.features[tag+'is_first_uppercase']=str(word.isupper())
		self.features[tag+'is_complete_uppercase']=str(word.upper()==word and word.isdigit()==False)
		self.features[tag+'is_numeric']=str(word.isdigit())
		self.features[tag+'is_alpha']=str(word.isalpha())
		self.features[tag+'is_alphanumeric']=str(bool((re.match('^(?=.*[0-9]$)(?=.*[a-zA-Z])',word))))
		self.features[tag+'length']=str(word)
		prefix_suffix_window=5
		for i in range(1,prefix_suffix_window+1):
			self.features[tag+'prefix_'+str(i)]=str(word[:i])
			self.features[tag+'suffix_'+str(i)]=str(word[-i:])
	def wordcontextFeatures(self,word,pos,lemma,tag):
		self.features[tag+'word']=str(word)
		self.features[tag+'pos']=pos
		self.features[tag+'lemma']=str(lemma)
			
def features(document,pos_list,lemma_list,index,useful_features):
	word=document[index]
	pos=pos_list[index]
	lemma=lemma_list[index]
	if index-1>0:
		prev_prev_word=document[index-2]
		prev_prev_pos=pos_list[index-2]
		prev_prev_lemma=lemma_list[index-2]
	else:
		prev_prev_word=''
		prev_prev_pos=''
		prev_prev_lemma=''
	if index>0:
		prev_word=document[index-1]
		prev_pos=pos_list[index-1]
		prev_lemma=lemma_list[index-1]
	else:
		prev_word=''
		prev_pos=''
		prev_lemma=''
	if index<len(document)-1:
		next_word=document[index+1]
		next_pos=pos_list[index+1]
		next_lemma=lemma_list[index+1]
	else:
		next_word=''
		next_pos=''
		next_lemma=''
	if index<len(document)-2:
		next_next_word=document[index+2]
		next_next_pos=pos_list[index+2]
		next_next_lemma=lemma_list[index+2]
	else:
		next_next_word=''
		next_next_pos=''
		next_next_lemma=''
	gen_features=GenerateFeatures()
	if useful_features[0]:
		gen_features.generateMorphologyFeatures(word,'')
		gen_features.generateMorphologyFeatures(prev_word,'prev_')
		gen_features.generateMorphologyFeatures(next_word,'next_')
		gen_features.generateMorphologyFeatures(prev_prev_word,'prev_prev_')
		gen_features.generateMorphologyFeatures(next_next_word,'next_next_')
	if useful_features[1]:
		gen_features.wordcontextFeatures(word,pos,lemma,'')
		gen_features.wordcontextFeatures(prev_word,prev_pos,prev_lemma,'prev_')
		gen_features.wordcontextFeatures(next_word,next_pos,next_lemma,'next_')
		gen_features.wordcontextFeatures(prev_prev_word,prev_prev_pos,prev_prev_lemma,'prev_prev_')
		gen_features.wordcontextFeatures(next_next_word,next_next_pos,next_next_lemma,'next_next_')
	return gen_features.features

def untag(document,option):
	#Passar de colunas para tuples
	if option!='predict':
		#return [str(word) for word,pos,tag in document],[pos for word,pos,tag in document],['' for word,pos,tag in document]
		return [str(word) for word,pos,lemma,tag in document],[pos for word,pos,lemma,tag in document],[lemma for word,pos,lemma,tag in document]
	else:
		#return [str(word) for word,pos in document],[pos for word,pos in document],['' for word,pos in document]
		return [str(word) for word,pos,lemma in document],[pos for word,pos,lemma in document],[lemma for word,pos,lemma in document]

def prepareData(tagged_documents,option,useful_features):
	#Construir o dataset a partir dos ficheiros -> Gerar features para usar no CRF
	X,y=[],[]
	for document in tagged_documents:
		words,postags,lemmas =untag(document,option)
		X.append([features(words,postags,lemmas,index,useful_features) for index in range(len(document))])
		if option!='predict':
			y.append([tag for word,pos,lemmas,tag in document])
	return X,y

def fromListToTuple(data):
	tuple_data=[]
	for l in data:
		tuple_data.append(tuple(l))
	return tuple_data

def fromTuplesToList(data,y):
	dataset=[]
	for i in range(len(data)):
		dataset.append([])
		for j in range(len(data[i])):
			list_j=list(data[i][j])
			list_j.append(y[i][j])
			dataset[i].append(list_j)
	return dataset

def save_model(nome,modelo):
	with open(nome,'wb') as f:
		pickle.dump(modelo,f,protocol=pickle.HIGHEST_PROTOCOL)

def load_model(nome):
	with open(nome,'rb') as f:
		 result = pickle.load(f)
	return result

def run_crf(data,load_file="harem.pickle"):
	useful_features=[True,True]
	X_test=fromListToTuple(data)
	X_teste,y_teste=prepareData([X_test],'predict',useful_features)
	crf = load_model(load_file)
	y_pred=crf.predict(X_teste)
	resultados = []
	for index,elem in enumerate(y_pred[0]):
		resultados.append(str(y_pred[0][index]))
	return resultados

def test_crf(train_file,test_file,model_name=""):
	l1 = [0.015625,0.03125,0.0625,0.125,0.25,0.5,1]
	l2 = [0.03125,0.0625,0.125,0.25,0.5,1,2,4,8,16]
	valores = []
	useful_features=[True,True]
	data=pandas.read_csv(train_file,sep="\t",header=None)
	X_dataset=fromListToTuple(data.iloc[:,[0,1,2,3]].values)
	X_teste,y_teste=prepareData([X_dataset],'test',useful_features)
	#X_teste, test = train_test_split(X_dataset, test_size=0.1)
	X_teste = pd.DataFrame(X_teste).transpose()
	y_teste = pd.DataFrame(y_teste).transpose()

	print(X_teste.shape)
	print(y_teste.shape)	
	X_teste_2 = X_teste
	y_teste_2 = y_teste
	crf = CRF(
		algorithm='lbfgs',
		#c1=0.0625,
		c1=1.0,
		#c2=0.5,
		c2=1.0,
		max_iterations=100,
		all_possible_transitions=False,
		all_possible_states=True,
		verbose=True
	)
	crf.fit(X_teste.values.tolist(), y_teste.values.tolist())
	y_pred=crf.predict(X_teste.values.tolist())
	labels=list(crf.classes_)
	save_model("NP_Final_Macro.pickle",crf)
	string = " "
	string += str(metrics.flat_classification_report(y_teste.values.tolist(), y_pred, labels=labels, digits=3))
	filename = "Results _.txt"
	print("$$$$$$$$$$$")
	print(filename)
	print("$$$$$$$$$$$")
	with open(filename,'a') as f:
		f.write(string)
	


if __name__ == '__main__':
	test_crf("prop_simplified.txt","prop_simplified.txt","")



import xmltodict
import re

class verb_normalizer:
	def __init__(self):

		self.declesion_exceptions=[]
		self.declesion_targets = []
		self.declesion_tags = []
		self.declesion_replacement = []
		self.declesion_suffix = ""
		self.declesion_prefix = ""
		self.declesion_re = []


		self.lexeme_exceptions=[]
		self.lexeme_targets = []
		self.lexeme_tags = []
		self.lexeme_replacement = []
		self.lexeme_rule_list=[]
		self.lexeme_suffix = ""
		self.lexeme_prefix = ""
		self.lexeme_re = []
		self.lexeme_re_normal = []


		self.conjugation_exceptions=[]
		self.conjugation_targets = []
		self.conjugation_tags = []
		self.conjugation_replacement = []
		self.conjugation_suffix = ""
		self.conjugation_prefix = ""
		self.conjugation_re = []
		self.conjugation_re_normal = []

	def compile_rules(self):
		self.compile_rules_declesions()
		self.compile_rules_lexemes()
		self.compile_rules_conjugation()

	def compile_rules_declesions(self):
		lista = [None] * len(self.declesion_targets)
		for index,elem in enumerate(self.declesion_targets):
			li = re.compile(self.declesion_prefix+elem+self.declesion_suffix)
			lista[index] = li
		self.declesion_re=lista

	def compile_rules_lexemes(self):
		lista = [None] * len(self.lexeme_targets)
		lista_normal = [None] * len(self.lexeme_targets)
		for index,elem in enumerate(self.lexeme_targets):
			li_normal = re.compile(elem+self.lexeme_suffix)
			li = re.compile(self.lexeme_prefix+elem+self.lexeme_suffix)
			lista[index] = li
			lista_normal[index] = li_normal
		self.lexeme_re=lista
		self.lexeme_re_normal = lista_normal

	def compile_rules_conjugation(self):
		lista = [None] * len(self.conjugation_targets)
		lista_normal = [None] * len(self.conjugation_targets)
		for index,elem in enumerate(self.conjugation_targets):
			li_normal = re.compile(elem+self.conjugation_suffix)
			li = re.compile(self.conjugation_prefix+elem)
			lista[index] = li
			lista_normal[index] = li_normal
		self.conjugation_re=lista
		self.conjugation_re_normal = lista_normal


	def verb_normalizer_load(self,file_name_conjugation,file_name_lexemes,file_name_declensions):
		#-----------------
		#Load regular declensions
		#-----------------
		with open(file_name_conjugation) as file:
			doc = xmltodict.parse( file.read())
			result = (doc["irregularverbconjugations"]["replacement"])
			
			'''
			Get the number of rules and set the size of the class variables
			'''
			verbconjugation_number_of_rules = len(result)
			self.conjugation_exceptions=[None]*verbconjugation_number_of_rules
			self.conjugation_targets = [None]*verbconjugation_number_of_rules
			self.conjugation_tags = [None]*verbconjugation_number_of_rules
			self.conjugation_replacement = [None]*verbconjugation_number_of_rules
			'''
			Check if the rule has any of the elements 
			'''
			for index,elem in enumerate(result):
				if '@target' in elem.keys():
					self.conjugation_targets[index] = elem["@target"]
				else:
					self.conjugation_targets[index] = ""
				if '@exceptions' in elem.keys():
					self.conjugation_exceptions[index] = elem["@exceptions"]
				else:
					self.conjugation_exceptions[index] = ""
				if '@tag' in elem.keys():
					self.conjugation_tags[index] = elem["@tag"]
				else:
					self.conjugation_tags[index] = ""
				if '#text' in elem.keys():
					self.conjugation_replacement[index] = elem["#text"]
				else:
					self.conjugation_replacement[index] = ""
			
			self.conjugation_suffix = (doc["irregularverbconjugations"]["suffix"])
			self.conjugation_prefix = (doc["irregularverbconjugations"]["prefix"])

		#-----------------
		#Load lexeme declensions
		#-----------------
		with open(file_name_lexemes) as file:
			doc = xmltodict.parse( file.read())
			result = (doc["regularverblexemes"]["replacement"])
			'''
			Get the number of rules and set the size of the class variables
			'''
			lexemedeclensions_number_of_rules = len(result)
			self.lexeme_exceptions=[None]*lexemedeclensions_number_of_rules
			self.lexeme_targets = [None]*lexemedeclensions_number_of_rules
			self.lexeme_tags = [None]*lexemedeclensions_number_of_rules
			self.lexeme_replacement = [None]*lexemedeclensions_number_of_rules
			'''
			Check if the rule has any of the elements 
			'''
			for index,elem in enumerate(result):
				if '@target' in elem.keys():
					self.lexeme_targets[index] = elem["@target"]
				else:
					self.lexeme_targets[index] = ""
				if '@exceptions' in elem.keys():
					self.lexeme_exceptions[index] = elem["@exceptions"]
				else:
					self.lexeme_exceptions[index] = ""
				if '@tag' in elem.keys():
					self.lexeme_tags[index] = elem["@tag"]
				else:
					self.lexeme_tags[index] = ""
				if '#text' in elem.keys():
					self.lexeme_replacement[index] = elem["#text"]
				else:
					self.lexeme_replacement[index] = ""
			self.lexeme_suffix = (doc["regularverblexemes"]["suffix"])
			self.lexeme_prefix = (doc["regularverblexemes"]["prefix"])

		#-----------------
		#Load conjugation declensions
		#-----------------
		with open(file_name_declensions) as file:
			doc = xmltodict.parse( file.read())
			result = (doc["regularverbdeclensions"]["replacement"])
			'''
			Get the number of rules and set the size of the class variables
			'''
			declensionsdeclensions_number_of_rules = len(result)
			self.declensions_exceptions=[None]*declensionsdeclensions_number_of_rules
			self.declensions_targets = [None]*declensionsdeclensions_number_of_rules
			self.declensions_tags = [None]*declensionsdeclensions_number_of_rules
			self.declensions_replacement = [None]*declensionsdeclensions_number_of_rules
			'''
			Check if the rule has any of the elements 
			'''
			for index,elem in enumerate(result):
				if '@target' in elem.keys():
					self.declensions_targets[index] = elem["@target"]
				else:
					self.declensions_targets[index] = ""
				if '@exceptions' in elem.keys():
					self.declensions_exceptions[index] = elem["@exceptions"]
				else:
					self.declensions_exceptions[index] = ""
				if '@tag' in elem.keys():
					self.declensions_tags[index] = elem["@tag"]
				else:
					self.declensions_tags[index] = ""
				if '#text' in elem.keys():
					self.declensions_replacement[index] = elem["#text"]
				else:
					self.declensions_replacement[index] = ""

			self.declesion_prefix = (doc["regularverbdeclensions"]["prefix"])


				
	def print_verb_normalizer(self):
		for i in range(len(self.declesion_targets)):
			print(i)
			print("---------\n")
			print("Target: " + self.declesion_targets[i])
			print("Tags: " + self.declesion_tags[i])
			print("Exceptions: " + self.declesion_exceptions[i])
			print("Replacement: " + self.declesion_replacement[i])
		print("---------------")
		print("---------------")
		print("---------------")
		print("---------------")

		for i in range(len(self.lexeme_targets)):
			print(i)
			print("---------\n")
			print("Target: " + self.lexeme_targets[i])
			print("Tags: " + self.lexeme_tags[i])
			print("Exceptions: " + self.lexeme_exceptions[i])
			print("Replacement: " + self.lexeme_replacement[i])
		print("---------------")
		print("---------------")
		print("---------------")
		print("---------------")

		for i in range(len(self.conjugation_targets)):
			print(i)
			print("---------\n")
			print("Target: " + self.conjugation_targets[i])
			print("Tags: " + self.conjugation_tags[i])
			print("Exceptions: " + self.conjugation_exceptions[i])
			print("Replacement: " + self.conjugation_replacement[i])


	def normalize_verb(self,token, tag):
		#print("Token de entrada: " + token)
		#choose biggest rule
		bigget_rule_size = 0
		#lemmatized_word = token.lower()
		lemmatized_word = ""
		'''
		Token to lowercase to match the rules
		'''
		normalization = token.lower()
		'''
		Check for every rule it the element matches the target,
		if the tags match and if the element is not an exception

		'''
		'''
		Conjugations (without  prefixes)
		'''

		traco = token[-1]
		if not(traco == "-"):
			traco = ""
		for index,elem in enumerate(self.conjugation_targets):
			desconta_valores=0
			desconta_valores2=0
			res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",self.conjugation_targets[index])
			tem_s = 1
			if(token[-1]=="s"):
				tem_s=0
			if(len(res)>0):
				for elem in res:
					desconta_valores += len(elem)-2
					desconta_valores2 += 2
			desconta_valores=len(self.conjugation_targets[index])-(desconta_valores+desconta_valores2+tem_s)
			word = normalization[:-desconta_valores]

			if( self.conjugation_re_normal[index].fullmatch(normalization)
				and tag.lower() in self.conjugation_tags[index].split("|")
				and not (normalization in self.conjugation_exceptions[index].split("|"))):
					
					'''
					Verificar se a regra a ser aplicada é a regra mais especifica para o caso 
					'''
					lemmatized_word = word + self.conjugation_replacement[index] + traco
					return(lemmatized_word)
					#print("RULE 1")

		'''
		Conjugations (with  prefixes)
		'''

		for index,elem in enumerate(self.conjugation_targets):
			#print("---"+token)
			desconta_valores = 0
			desconta_valores2 = 0
			#print(self.conjugation_targets[index])
			res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",self.conjugation_targets[index])
			tem_s = 1
			if(token[-1]=="s"):
				tem_s=0
			if(len(res)>0):
				for elem in res:
					desconta_valores += len(elem)-2
					desconta_valores2 += 2
			desconta_valores=len(self.conjugation_targets[index])-(desconta_valores+desconta_valores2+tem_s)
			word = normalization[:-desconta_valores]
			

			if( self.conjugation_re[index].fullmatch(normalization)
				and tag.lower() in self.conjugation_tags[index].split("|")
				and not (normalization in self.conjugation_exceptions[index].split("|"))):

					'''
					Verificar se a regra a ser aplicada é a regra mais especifica para o caso 
					'''

					lemmatized_word = word + self.conjugation_replacement[index] + traco
					return(lemmatized_word)
					#print("RULE 1")
		'''
		Regular ver lexemes (without prefixes)
		'''
		for index,elem in enumerate(self.lexeme_targets):
			desconta_valores = 0
			desconta_valores2 = 0
			#print(self.lexeme_targets[index])
			res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",self.lexeme_targets[index])
			tem_s = 1
			if(token[-1]=="s"):
				tem_s=0
			if(len(res)>0):
				for elem in res:
					desconta_valores += len(elem)-2
					desconta_valores2 += 2
			desconta_valores=len(self.lexeme_targets[index])-(desconta_valores+desconta_valores2+tem_s)
			word = normalization[:-desconta_valores]
			#print("-----"+elem+"----"+word)
			if( self.lexeme_re_normal[index].fullmatch(normalization)
				and tag.lower() in self.lexeme_tags[index].split("|")
				and not (normalization in self.lexeme_exceptions[index].split("|"))):

					
					'''
					Verificar se a regra a ser aplicada é a regra mais especifica para o caso 
					'''

					lemmatized_word = self.lexeme_replacement[index] + traco
					return(lemmatized_word)
		'''
		Regular ver lexemes (with prefixes)
		'''
		for index,elem in enumerate(self.lexeme_targets):
			desconta_valores = 0
			desconta_valores2 = 0
			#print(self.lexeme_targets[index])
			res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",self.lexeme_targets[index])
			tem_s = 1
			if(token[-1]=="s"):
				tem_s=0
			if(len(res)>0):
				for elem in res:
					desconta_valores += len(elem)-2
					desconta_valores2 += 2
			desconta_valores=len(self.lexeme_targets[index])-(desconta_valores+desconta_valores2+tem_s)
			word = normalization[:-desconta_valores]
			#print("-----"+elem+"----"+word)
			if( self.lexeme_re[index].fullmatch(normalization)
				and tag.lower() in self.lexeme_tags[index].split("|")
				and not (normalization in self.lexeme_exceptions[index].split("|"))):

					
					'''
					Verificar se a regra a ser aplicada é a regra mais especifica para o caso 
					'''
					lemmatized_word = word + self.lexeme_replacement[index] + traco
					return(lemmatized_word)

		for index,elem in enumerate(self.declesion_targets):
			desconta_valores = 0
			desconta_valores2 = 0
			res = re.findall(r"\[(?:\w|àáãâéêíóõôúç\-)*\]",self.declesion_targets[index])
			tem_s = 1
			if(token[-1]=="s"):
				tem_s=0
			if(len(res)>0):
				for elem in res:
					desconta_valores += len(elem)-2
					desconta_valores2 += 2
			desconta_valores=len(self.declesion_targets[index])-(desconta_valores+desconta_valores2+tem_s)
			word = normalization[:-desconta_valores]
			if( self.declesion_re[index].fullmatch(normalization)
				and tag.lower() in self.declesion_tags[index].split("|")
				and not (normalization in self.declesion_exceptions[index].split("|"))):

					'''
					Verificar se a regra a ser aplicada é a regra mais especifica para o caso 
					'''
					lemmatized_word = word+ self.declesion_replacement[index] + traco
					return(lemmatized_word)
		return(lemmatized_word)

	
	def lexemes_sufixos(self):
		teste = "agigant[eiao]"
		prog = re.compile(teste)
		result = prog.match("agigantaaaaa")
		return result

